@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Joel Rosdahl
+ * Copyright (C) 2010-2011 Joel Rosdahl
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -99,6 +99,14 @@ lockfile_acquire(const char *path, unsigned staleness_limit)
 			goto out;
 		}
 		cc_log("lockfile_acquire: symlink %s: %s", lockfile, strerror(errno));
+		if (errno == EPERM) {
+			/*
+			 * The file system does not support symbolic links. We have no choice but
+			 * to grant the lock anyway.
+			 */
+			acquired = true;
+			goto out;
+		}
 		if (errno != EEXIST) {
 			/* Directory doesn't exist or isn't writable? */
 			goto out;
